/* ***** BEGIN LICENSE BLOCK *****

 * Version: MPL 1.1/GPL 2.0/LGPL 2.1
 * The Original Code is bookmarkshome
 *
 * The Initial Developer of the Original Code is Jeroen Groenenboom.
 *
 * Contributor(s):
 *   Dimiter Dimitrov <dimiter@blue-edge.bg>
 *   Josh Powell <Josh@ki4bbo.org> 
 *   Jean-Bernard Marcon <jean-bernard.marcon@laposte.net>
 *
 * ***** END LICENSE BLOCK ***** */

function bmhGetNavigatorWindow() { 
    // let bmh fail if browser window cannot be obtained
    return self.QueryInterface( Components.interfaces.nsIInterfaceRequestor )
	.getInterface( Components.interfaces.nsIWebNavigation )
	.QueryInterface( Components.interfaces.nsIDocShellTreeItem )
	.rootTreeItem
	.QueryInterface( Components.interfaces.nsIInterfaceRequestor )
	.getInterface( Components.interfaces.nsIDOMWindow );
}

var bmhLib = bmhGetNavigatorWindow().bmhLib;

// constructor for column
function bmhColumnTd() {
    this.node = document.createElement( "td" );
    this.offsetNode = document.createElement( "div" );
    this.node.appendChild( this.offsetNode );
}

bmhColumnTd.prototype = {

isEmpty: function() {
	     return this.offsetNode == this.node.firstChild;
	 },


getOffset: function() {
	       return this.offsetNode.offsetTop;
	   },

add: function( folderDiv ) {
	 this.node.insertBefore( folderDiv.node, this.offsetNode );
     }
}

function bmhFolderDiv( folder, index ) {
    // constructor for folder
    this.node = document.createElement( "div" );
    this.node.setAttribute( "id", "folder" + index );
    this.node.setAttribute( "class", "folder" );
    this.type = folder.type;
    this.index = index;
    // hover div
    var hoverFolderH1 = bmhLib.getRDFPrefFromArray( folder.ancestors, location.href );
    if( folder.ancestors.length > 1 ) {
	// don't show root folder
	hoverFolderH1 = hoverFolderH1.replace( /^[^\|]*\|/, "" );
    }
    hoverFolderH1 = decodeURI( hoverFolderH1.replace( /\s*\|\s*/g, " > " ));
    var hoverDiv = this.createHoverDiv( hoverFolderH1, folder.description );
    // folder h1
    var folderH1 = document.createElement( "h1" );
    this.node.appendChild( folderH1 );
    // folder anchor
    var folderA = this.createAnchor( "javascript:void(0)" );
    folderA.removeAttribute( "target" );
    folderA.setAttribute( "class", "headerLink" );
    var listId = "folderList" + this.index;
    folderA.setAttribute( "onclick", "void(bmhCheck('openFolder','" + listId + "'))" );
    this.createHeaderText( folder.ancestors, folderA ); 
    folderH1.appendChild( folderA );
    this.node.appendChild( hoverDiv );
    // folder ul
    this.folderUl = document.createElement( "ul" );
    this.folderUl.setAttribute( "id", listId );
    this.node.appendChild( this.folderUl );
}


bmhFolderDiv.prototype = {

isEmptyFolder: function() {
		   return( this.type == "Folder"
			   ? this.folderUl.getElementsByTagName( "a" ).length == 0
			   : false );
	       },

isEmptyLivemark: function() {
		     return( this.type == "Livemark"
			     ? this.folderUl.getElementsByTagName( "a" ).length <= 1
			     : false );
		 },


createHeaderText: function( ancestors, folderA ) {
		      // create folder header based on maximum header lines pref
		      var headerArray = eval( ancestors.toSource() );
		      var bmhTop = headerArray.length - bmhLib.getRDFPref( "maxHeaderLines", location.href );
		      var i = bmhTop;
		      if( bmhTop > 1 ) {
			  // at least one ancestor won't be displayed
			  headerArray[bmhTop] = "> ".concat( headerArray[bmhTop] );
			  i--;
			  // append '>' for other ancestors that won't be displayed
			  for( ; i > 1; i-- ) {
			      headerArray[bmhTop] = ">".concat( headerArray[bmhTop] );
			  }
		      }
		      // bmhTop <= 1 -> all ancestors (except root) will be displayed
		      i = ( bmhTop <= 1 ? 1 : bmhTop );
		      // root ancestor only displayed for root folder
		      if( ancestors.length == 1 ) {
			  i = 0;
		      }
		      var folderName, headerBr, span;
		      for( ; i < headerArray.length; i++ ) {
			  folderName = document.createTextNode( this.processName( headerArray[i] ));	
			  span = document.createElement( "span" );
			  folderA.appendChild( span );
			  span.appendChild( folderName );
		      }
		  },

createAnchor: function( url ) {
		  var folderA = document.createElement( "a" );
		  folderA.setAttribute( "href", url );
		  folderA.setAttribute( "onmouseover", "showHoverInfo(this)" );
		  folderA.setAttribute( "onmouseout", "hideHoverInfo(this)" );
		  // reset to headerLink by constructor
		  if( this.type == "Livemark" ) {
		      folderA.setAttribute( "class", "liveAnchor" );
		  }
		  else if( this.type == "Folder" ) {
		      folderA.setAttribute( "class", "folderAnchor" );
		  }
		  if( bmhLib.getRDFPref( "newWindowLinks", location.href )) { 
		      folderA.setAttribute( "target", "_blank" );
		  }
		  return folderA;
	      },

createHoverDiv: function( name, desc ) {
		    var hoverDiv = document.createElement( "div" );
		    //var hoverDiv = document.createElement( "span" );
		    hoverDiv.setAttribute( "class", "hoverDiv" );
		    hoverDiv.setAttribute( "style", "visibility:hidden;" );
		    var displayIndex = bmhLib.getRDFPref( "tooltipDisplay", location.href );
		    if( displayIndex == 0 ) {
			hoverDiv.style.display = "block ! important";
		    }
		    else if( displayIndex == 2 ) {
			hoverDiv.style.display = "none ! important";
		    }
		    else if( this.type == "Livemark" ) {
			// displayIndex == 1
			hoverDiv.style.display = "block ! important";
		    }
		    else {
			hoverDiv.style.display = "none ! important";
		    }
		    var hoverHeader = document.createElement( "h1" );
		    // may contain html
		    hoverHeader.innerHTML = name;
		    hoverDiv.appendChild( hoverHeader );
		    var descriptionSpan = document.createElement( "span" );
		    // may contain html
		    descriptionSpan.innerHTML = desc;
		    hoverDiv.appendChild( descriptionSpan );
		    return hoverDiv;
		},

add: function( name, url, description, icon ) {
	 // folder li
	 var folderLi = document.createElement( "li" );
	 this.folderUl.appendChild( folderLi );
	 // folder a
	 var folderA = this.createAnchor( url ); 
	 folderLi.appendChild( folderA );
	 var aName = this.processName( name );
	 // text a
	 var aText = document.createTextNode( aName );
	 // icons, list style icons doesn't work
	 var image;
	 if(( icon != "" ) && ( this.type == "Folder" )) {
	     image = document.createElement( "img" );
	     image.setAttribute( "src", icon );
	     image.setAttribute( "alt", "" );
	     image.setAttribute( "class", "normalIcon" );
	     image.style.display = ( 
		     bmhLib.getRDFPref( "hideIcons", location.href ) ? "none ! important" : "inline ! important" );
	     folderA.appendChild( image );
	 }
	 folderA.appendChild( aText ); 
	 // add hover item
	 var hoverDiv = this.createHoverDiv( name, description ); 
	 folderLi.appendChild( hoverDiv );
     },

addSeparator: function() {
		  var folderLi = document.createElement( "li" );
		  folderLi.setAttribute( "class", "separator" );
		  this.folderUl.appendChild( folderLi );
	      },

processName: function( name ) {
		 var displayName = name;
		 var reg1 = /^(?:\w+:[\/\\]{1,3}(?:www\.)?)/; 
		 var reg2 = /.{2}$/;	
		 var max = bmhLib.getRDFPref( "maxNameLength", location.href );
		 if( displayName.length > max ) {
		     // remove protocol of url
		     displayName = displayName.replace( reg1, "" );
		     if( displayName.length > max ) {
			 displayName = displayName.substring( 0, max );
			 // show that name is longer than portrayed
			 displayName = displayName.replace( reg2, ".." );
		     }
		 }
		 return displayName;
	     }
}

function bmhFolder( node, ancestors, desc, type, httpReq ) {
    this.rdfNode = node;
    // ancestors include name of folder
    this.ancestors = ancestors;
    this.description = desc;
    this.type = type;
    this.httpReq = httpReq;
    if( httpReq ) {
	httpReq.ancestors = ancestors;
    }
}


var bmh = {
folderDivs: [],
	    httpRequests: [],
	    httpSuccessCounter: 0,
	    httpErrorCounter: 0,

	    main: function() {
		
		var statusDiv = document.getElementById( "statusDiv" );
		try {
		    bmh.localize();
		    // rdf 
		    var curName = bmhLib.getProperty( bmhLib.bmRoot, bmhLib.NameArc, bmhLib.BMDS );
		    var curFolder = new bmhFolder( bmhLib.bmRoot, [curName], "", "Folder", null );
		    var folderStack = new Array( curFolder );
		    var inOrExcludedFolders = bmhLib.getArrayFromRDFPref( "inOrExcludedFolders", location.href );
		    // search engines
		    bmh.createEngineMenu();
		    bmh.showHideSearchBar();
		    // add style 
		    bmh.addStyle();
		    // columns
		    var columns = bmhLib.getRDFPref( "nrOfColumns", location.href ); 
		    var columnTds = bmh.createColumns( columns );
		    // misc vars 
		    var folderDiv, tempFolders, isExcluded, folderIndex = -1, httpRequest;
		    var excludeIndex = bmhLib.getRDFPref( "excludeIndex", location.href );
		    // make page
		    while( folderStack.length > 0 ) {
			curFolder = folderStack.pop();
			folderIndex++;
			if( curFolder.httpReq ) {
			    curFolder.httpReq.folderIndex = folderIndex;
			}
			folderDiv = new bmhFolderDiv( curFolder, folderIndex );
			tempFolders = bmh.parseFolder( curFolder, folderDiv, 
				excludeIndex, inOrExcludedFolders ); 
			isExcluded = bmhLib.isExcludedFolder( curFolder.ancestors, excludeIndex, inOrExcludedFolders );
			if( isExcluded || folderDiv.isEmptyFolder() ) {
			    // folderIndex in id of folder should match index in bmh.folderDivs
			    folderIndex--;
			}
			else {
			    bmh.tessellate( columnTds, folderDiv );
			    bmh.folderDivs.push( folderDiv );
			}
			folderStack = folderStack.concat( tempFolders ); 
		    }
		    // remove empty columns
		    var theRow = document.getElementById( "theRow" );
		    for( var i = 0; i < columnTds.length; i++ ) {
			if( columnTds[i].isEmpty() ) {
			    theRow.removeChild( columnTds[i].node );
			}
		    }
		    if( bmh.httpRequests.length == 0 ) {
			// don't rebuild, just create static page
			statusDiv.textContent = "F";
			bmh.createStaticPage();
		    }
		    else {
			for( i = 0; i < bmh.httpRequests.length; i++ ) {
			    bmh.httpGet( bmh.httpRequests[i] );
			}
		    }
		}
		catch( e ) {
		    statusDiv.textContent = "E";
		    throw( e );
		}
	    },

parseFolder: function( folder, folderDiv, excludeIndex, inOrExcludedFolders ) {
		 bmhLib.RDFC.Init( bmhLib.BMDS, folder.rdfNode );
		 var enumerator = bmhLib.RDFC.GetElements();
		 var foundFolders = new Array();
		 var curNode, curType, curName, curDesc, curURL, curURL, curIcon, 
		     curNodeAncestors, httpRequest;
		 while( enumerator.hasMoreElements() ) {
		     curNode = enumerator.getNext();
		     curType = bmhLib.getProperty( curNode, bmhLib.typeArc, bmhLib.BMDS )
			 .split( "#" )[1]; 
		     curName = bmhLib.getProperty( curNode, bmhLib.NameArc, bmhLib.BMDS );
		     curDesc = bmhLib.getProperty( curNode, bmhLib.DescriptionArc, bmhLib.BMDS );
		     if( curType == "Bookmark" ) {
			 // Bookmark
			 curURL = bmhLib.getProperty( curNode, bmhLib.URLArc, bmhLib.BMDS ); 
			 curIcon = bmhLib.getProperty( curNode, bmhLib.IconArc, bmhLib.BMDS );
			 // icon in link of bmh page is not picked up by FF 
			 if( bmhLib.getWhat( curURL ) == "bookmarkshome" ) {
			     curIcon = document.getElementsByTagName( "link" )[0].getAttribute( "href" );
			 }
			 folderDiv.add( curName, curURL, curDesc, curIcon );
		     }
		     else if( curType == "Folder" ) {
			 // Folder
			 curNodeAncestors = folder.ancestors.concat( [curName] );
			 foundFolders.unshift( new bmhFolder( curNode, curNodeAncestors, curDesc, curType, null ));
		     }
		     else if( curType == "Livemark" ) {
			 curNodeAncestors = folder.ancestors.concat( [curName] );
			 if( ! bmhLib.isExcludedFolder( curNodeAncestors, excludeIndex, inOrExcludedFolders )) {
			     // livemark contains no other folders, so if excluded, don't push it on folderStack
			     curNodeAncestors = folder.ancestors.concat( [curName] );
			     httpRequest = new XMLHttpRequest();
			     httpRequest.feedURL = bmhLib.getProperty( curNode, bmhLib.FeedURLArc, bmhLib.BMDS );
			     // add http request
			     bmh.httpRequests.push( httpRequest );
			     foundFolders.unshift( 
				     new bmhFolder( curNode, curNodeAncestors, curDesc, curType, httpRequest ));
			 }
		     }
		     else if( curType == "BookmarkSeparator" ) {
			 folderDiv.addSeparator();
		     }
		 }
		 return foundFolders;
	     },


tessellate: function( columnTds, folderDiv ) {
		var index = bmh.smallestColumn( columnTds );
		columnTds[index].add( folderDiv );
	    },

createColumns:function( nr ) {
		  var theRow = document.getElementById( "theRow" ); 
		  for( var i = theRow.childNodes.length - 1; i >= 0; i-- ) {
		      theRow.removeChild( theRow.childNodes[i] );
		  }
		  var columnTds = new Array( nr );
		  for( i = 0; i < columnTds.length; i++ ) {
		      columnTds[i] = new bmhColumnTd();
		      theRow.appendChild( columnTds[i].node );
		  }
		  return columnTds;
	      },

rebuild: function() {
	     var nrOfColumns = document.getElementById( "theRow" ).childNodes.length;
	     var columnTds = bmh.createColumns( nrOfColumns );
	     for( i = 0; i < bmh.folderDivs.length; i++ ) {
		 bmh.tessellate( columnTds, bmh.folderDivs[i] ); 
	     }
	     bmh.folderDivs = null;
	 },

checkHttpRequests: function() {
		       var handledRequests = bmh.httpErrorCounter + bmh.httpSuccessCounter;
		       if(( bmh.httpRequests.length - handledRequests ) == 0 ) {
			   bmh.httpRequests = null;
			   var statusDiv = document.getElementById( "statusDiv" );
			   if( bmh.httpErrorCounter > 0 ) {
			       statusDiv.textContent = "E";
			   }
			   else {
			       statusDiv.textContent = "F";
			   }
			   if( bmh.httpSuccessCounter > 0 ) {
			       bmh.rebuild();
			   }
			   bmh.createStaticPage();
		       }
		   },

httpGet: function( req ) {
	     try {
		 bmh.httpRequestCounter++;
		 req.open( "GET", req.feedURL );
		 req.onload = bmh.httpLoaded;
		 req.onerror = bmh.httpError;
		 req.setRequestHeader( "User-Agent", CommonFunc.USER_AGENT );
		 req.overrideMimeType( "application/xml" );
		 req.send( null );
	     }
	     catch( e ) {
		 bmh.httpErrorCounter++;
		 bmh.checkHttpRequests();
	     }
	 },

httpError: function( event ) {
	       event.target.abort();
	       bmh.httpErrorCounter++;
	       bmh.checkHttpRequests();
	   },

httpLoaded: function( event ) {
		try {
		    var req = event.target;
		    var feed = new Feed( req.responseXML );
		    var folder = new bmhFolder( null, req.ancestors, feed.getHeaderDesc(), "Livemark", null );
		    var folderDiv = new bmhFolderDiv( folder, req.folderIndex );
		    var title, link, content; 
		    for( i = 0; i < feed.items.length; i++ ) {
			folderDiv.add( feed.items[i].getTitle(), feed.items[i].getLink(), feed.items[i].getContent(), 
				"", "Livemark" );
		    }
		    var oldFolderNode = document.getElementById( "folder" + req.folderIndex );
		    oldFolderNode.parentNode.replaceChild( folderDiv.node, oldFolderNode ); 
		    bmh.folderDivs[req.folderIndex] = folderDiv;
		    bmh.httpSuccessCounter++;
		}
		catch( e ) {
		    bmh.httpErrorCounter++;
		}
		bmh.checkHttpRequests();
	    },

createStaticPage: function() {
		      // next reload static only if no exceptions
		      var bmhFile = bmhLib.getBmhFile( location.href );
		      if( bmhFile ) {
			  var saveDoc = document.documentElement.cloneNode( true );
			  var bodies = saveDoc.getElementsByTagName( "body" );
			  bodies[0].setAttribute( "onload", "displayProperly()" );
			  var statusDiv = saveDoc.getElementsByTagName( "div" )[0];
			  statusDiv.textContent = "D";
			  //var docStr = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01//EN"' + 
			  //' "http://www.w3.org/TR/html4/strict.dtd">\n';
			  var docStr = '<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN"' + 
			  ' "http://www.w3.org/TR/1999/REC-html401-19991224/loose.dtd">\n';
			  docStr += ("<html>\n" + saveDoc.innerHTML + "\n</html>");
			  var fileStream = Components.classes["@mozilla.org/network/file-output-stream;1"]
			      .createInstance( Components.interfaces.nsIFileOutputStream );
			  fileStream.init( bmhFile, 0x20|0x02|0x08, 0644, 0 );
			  var utf8Stream = Components.classes["@mozilla.org/intl/converter-output-stream;1"]
			      .createInstance( Components.interfaces.nsIConverterOutputStream );
			  utf8Stream.init( fileStream, null, 0, '?' );
			  utf8Stream.writeString( docStr );
			  utf8Stream.close();
			  if( bmh.httpErrorCounter == 0 ) {
			      // check observer component
			      var observer = Components.classes["@jeroen/bmhBookmarksObserverService;1"]
				  .getService( Components.interfaces.nsIRDFObserver );
			      // static page creation sucessful, no empty livemarks, observer exists
			      // next reload can be static page
			      bmhLib.setDynamicsPref( location.href, false );
			  }
		      }
		  },


localize: function() {
	      // insert element with text for alert statement in script of document
	      var alertSpan = document.createElement( "span" );
	      alertSpan.setAttribute( "id", "alertSpan" );
	      alertSpan.textContent = bmhLib.getLocaleString( "NotAvailableString" );
	      alertSpan.style.display = "none";
	      document.getElementById( "otherLinks" ).appendChild( alertSpan );
	      // submit button
	      var submitInput = document.getElementById( "submitInput" );
	      submitInput.value = bmhLib.getLocaleString( "SearchButtonString" );
	      // text of bottom 
	      var readmeLink = document.getElementById( "readmeLink" );
	      readmeLink.textContent = bmhLib.getLocaleString( "ReadmeString" );
	      var homeLink = document.getElementById( "homeLink" );
	      homeLink.textContent = bmhLib.getLocaleString( "HomePageString" );
	      var optionsLink = document.getElementById( "optionsLink" );
	      optionsLink.textContent = bmhLib.getLocaleString( "OptionsString" );
	      var refreshLink = document.getElementById( "refreshLink" );
	      refreshLink.textContent = bmhLib.getLocaleString( "RefreshString" );
	      var uploadLink = document.getElementById( "uploadLink" );
	      uploadLink.textContent = bmhLib.getLocaleString( "UploadString" );
	  },

showHideSearchBar: function() {
		       var searchForm = document.getElementById( "searchForm" );
		       var external = bmhLib.getRDFPref( "externalEngines", location.href );
		       searchForm.setAttribute( "class", ( external ? "blank" : "self" )); 
		       if( bmhLib.getRDFPref( "hideSearchEngines", location.href )) {
			   searchForm.style.display = "none";
		       }
		       if( searchForm.style.display != "none" ) {
			   document.getElementById( "queryInput" ).focus();
		       }
		   },

smallestColumn: function( columns ) {
		    var index = 0;
		    for(var i = 1; i < columns.length; i++ ) {
			if( columns[i].getOffset() < columns[index].getOffset() ) {
			    index = i;
			}
		    }
		    return index;
		},

showDefaultEngine: function( index ) {
		       document.getElementById( "engineSelect" ).selectedIndex = index;
		   },

openFolder: function( listId ) {
		var links = document.getElementById( listId ).getElementsByTagName( "a" );
		// open in tabs as default
		var openIn = bmhLib.getRDFPref( "openFolderIn", location.href ) || 0;
		if( openIn == 0 ) {
		    var browser = bmhGetNavigatorWindow().getBrowser();
		    for( var i = 0; i < links.length; i++ ) { 
			browser.loadOneTab( links[i].getAttribute( "href" ));
		    }
		}
		else if( openIn == 1 ) {
		    // open in new windows
		    for( i = 0; i < links.length; i++ ) {
			bmhGetNavigatorWindow()
			    .openNewWindowWith( links[i].getAttribute( "href" ), null, null );
		    }
		}
		// if 2, do nothing
	    },

addStyle: function() { 
	      // let bmh fail if style cannot be added
	      var dfltStyleName = bmhLib.getRDFPref( "styleName", location.href );
	      var defaultStyle = bmhLib.RDF.GetResource( bmhLib.stylesNS + dfltStyleName );
	      var defaultSheet = bmhLib.getProperty( defaultStyle, bmhLib.sheetArc, bmhLib.stylesDS );
	      if( ! defaultSheet ) {
		  // get first sheet available
		  var stylesList = bmhLib.RDF.GetResource( bmhLib.stylesNS + "stylesList" );
		  bmhLib.RDFC.Init( bmhLib.stylesDS, stylesList );
		  var stylesEnum = bmhLib.RDFC.GetElements();
		  if( stylesEnum.hasMoreElements() ) {
		      defaultStyle = stylesEnum.getNext();
		      defaultSheet = bmhLib.getProperty( defaultStyle, bmhLib.sheetArc, bmhLib.stylesDS );
		  }
	      }
	      // insert stylesheet in document
	      var styleTag = document.getElementsByTagName( "style" )[0];
	      styleTag.innerHTML = bmhLib.formatSheet( defaultSheet );
	  },

openSidebar: function() {
		 try {
		     var mainDoc = bmhLib.getNavigatorDocument();
		     var bmhBroadcasterId = "viewBmhPrefs";
		     var bmhBroadcaster = mainDoc.getElementById( bmhBroadcasterId );
		     var sidebar = mainDoc.getElementById( "sidebar" );
		     var sidebarBox = mainDoc.getElementById( "sidebar-box" );
		     var sidebarTitle = mainDoc.getElementById( "sidebar-title" );
		     var sidebarSplitter = mainDoc.getElementById( "sidebar-splitter" );
		     var sidebarBroadcasters = mainDoc.getElementsByAttribute( "group", "sidebar" );
		     for ( var i = 0; i < sidebarBroadcasters.length; i++ ) {
			 sidebarBroadcasters[i].removeAttribute( "checked" );
		     }
		     bmhBroadcaster.setAttribute( "checked", "true" );
		     /*
			if( sidebarBox.hidden ) {
			sidebarBox.hidden = false;
			sidebarSplitter.hidden = false;
			}
		      */
		     var url = bmhBroadcaster.getAttribute( "sidebarurl" );
		     var title = bmhBroadcaster.getAttribute( "sidebartitle" );
		     if (!title) {
			 title = bmhBroadcaster.getAttribute( "label" );
		     }
		     sidebar.setAttribute( "src", url );
		     sidebarBox.setAttribute( "src", url );
		     sidebarBox.setAttribute( "sidebarcommand", bmhBroadcaster.id );
		     sidebarTitle.setAttribute( "value", title );
		     if( sidebarBox.hidden ) {
			 sidebarBox.hidden = false;
			 sidebarSplitter.hidden = false;
		     }
		     else {
			 sidebar.contentWindow.bmhOptionsOnload();
		     }
		     //return sidebar;
		 }
		 catch( e ) {
		     // just try again
		 }
	     },

createEngineMenu: function() {
		      try {
			  var engineTargets = bmhLib.ISDS.GetTargets( bmhLib.rootEngine, bmhLib.childArc, true );
			  var curNode, curName, curIcon, searchURL, curOption, hiddenIndex, curTextNode;
			  var engineSelect = document.getElementById( "engineSelect" );
			  var dfltEngineValue = bmhLib.getRDFPref( "engineURL", location.href );
			  var hiddenEngines = bmhLib.getArrayFromRDFPref( "hiddenEngines", location.href );
			  var dfltOption = null;
			  var newHiddenEngines = [];
			  while ( engineTargets.hasMoreElements()) {
			      curNode = engineTargets.getNext().QueryInterface( Components.interfaces.nsIRDFResource );
			      curName = bmhLib.getProperty( curNode, bmhLib.NameArc, bmhLib.ISDS );
			      curIcon = bmhLib.getProperty( curNode, bmhLib.IconArc, bmhLib.ISDS );
			      try {
				  searchURL = bmhLib.ISEARCHSVC.GetInternetSearchURL( 
					  curNode.Value, "bmh", 0, 0, {value:0} );
			      }
			      catch( e ) {
				  searchURL = "invalid searchurl";
			      }
			      curOption = document.createElement( "option" );
			      curOption.value = curNode.Value + "|" + curIcon + "|" + searchURL;
			      // determine default engine
			      if( curNode.Value == dfltEngineValue ) {
				  dfltOption = curOption;
			      }
			      hiddenIndex = bmhLib.binarySearch( curNode.Value, hiddenEngines );
			      // check if engine is hidden
			      if( hiddenIndex >= 0 ) {
				  curOption.style.display = "none";
				  // newHidden is hidden without removed engines
				  newHiddenEngines = newHiddenEngines.concat( hiddenEngines.splice( hiddenIndex, 1 ));
			      }
			      curTextNode = document.createTextNode( curName );
			      curOption.appendChild( curTextNode );
			      engineSelect.appendChild( curOption );
			  }
			  bmhLib.setRDFPref( "hiddenEngines", 
				  bmhLib.getRDFPrefFromArray( newHiddenEngines ), location.href ); 
			  bmhLib.selectEngine( dfltOption );
		      }
		      catch( e ) {
			  // no engines, to bad
		      }
		  }
}



